%{
// Scanner module for usage with the bison generated parser.
//
// Copyright 2001, KUN.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

// $Id: scanner.l,v 1.7 2003/09/27 00:46:42 murphy Exp $

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif // HAVE_CONFIG_H

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#else // HAVE_CONFIG_H
#include <stdlib.h>
#endif // HAVE_CONFIG_H

using namespace std;
#include <string.h>

#include "parserinterface.h"
#include "parser.h"
#include <str_util.h>

char* str_input_cpy(char* src, int len);
static int col_nr = 0;

#define YY_NO_UNPUT

%}

%option yylineno
%option noyywrap

LAYOUT      [ \t\r\015\032]+
COMMENT     #.*
NUMBER      0|[1-9][0-9]*
STR_CHARS   ((\\.)|[ !#-\[\]-~\177-\377])+
STRING      \"{STR_CHARS}\"
ORNAMENT    [\+\-\?@\^~_]
LETTER      [a-zA-Z\177-\377]
LETMENTS    ({LETTER}|{ORNAMENT})*{LETTER}({LETTER}|{ORNAMENT})*
IDPART      {LETMENTS}+


%%

\n              { col_nr = 1; return NEWLINE; }
{COMMENT}   	{ col_nr += yyleng; }
{LAYOUT}    	{ col_nr += yyleng; }
{NUMBER}    	{
                    col_nr += yyleng;
                    yylval.number = atoi(yytext);
                    return NUMBER;
                }
{STRING}        {
                    col_nr += yyleng;
                    yylval.chars = str_input_cpy(yytext, yyleng);
                    return STRING;
                }
{IDPART}	{
                    col_nr += yyleng;
                    yylval.string_ptr = new string(yytext);
                    return IDPART;
                }
::          	{ col_nr += yyleng; return DOUBLECOLON; }
:           	{ col_nr += yyleng; return COLON; }
\.{NUMBER}      { col_nr += yyleng; return PERIODNUMBER; }
\.          	{ col_nr += yyleng; return PERIOD; }
,           	{ col_nr += yyleng; return COMMA; }
;           	{ col_nr += yyleng; return SEMICOLON; }
\(          	{ col_nr += yyleng; return OPENPAR; }
\)          	{ col_nr += yyleng; return CLOSEPAR; }
\|          	{ col_nr += yyleng; return VBAR; }
&               { col_nr += yyleng; return AMPERSAND; }
\/              { col_nr += yyleng; return SLASH; }
\{              { col_nr += yyleng; return OPENGUARD; }
\}              { col_nr += yyleng; return CLOSEGUARD; }
\[              { col_nr += yyleng; return OPENOPT; }
\]              { col_nr += yyleng; return CLOSEOPT; }
.           	{ col_nr += yyleng; return yytext[0]; }

%%

int get_line_nr()
{
    return yylineno;
}

int get_col_nr()
{
    return col_nr;
}

char* str_input_cpy(char* src, int len)
{
    char* new_src;
    char* dst;

    new_src = strdup(src);
    dst = (char*) malloc(len);
    memset(dst, 0, len);
    dst = lexeme_copy(dst, new_src, len);
    free(new_src);

    return dst;
}

void reset_scanner_counters()
{
    yylineno = 1;
    col_nr = 1;
}

